#include "ti83plus.inc"
.org 4000h
;***********************************************************************
;*  TITLE: "MirageOS" APPS HEADER FILE                         	       *
;*                                                                     *
;*  AUTHOR: Dan Englender, Jason Kovacs                                *
;*                                                                     *
;* =================================================================== *
;* REVISION HISTORY:                                                   *
;*  1. MM/DD/YY: Initial Build for TI-83Plus SDK developers  - XXX     *
;*  2. 09/15/99: Elaborate w/comments; provide developers with a sample*
;*               solution to 5-digit keyfiles as well as renaming the  *
;*               name of their developed application(s)    - PJF       *
;*  3. 10/05/99  More comments/aid to developers - PJF                 *
;*  4. 11/29/99  Added text on signings to remodify header             *
;*                                                                     *
;***********************************************************************
;
; <place the "ti83plus.inc" file of your program before the apps header>
; <place any additional program directives, eg:  extern or public, etc.
;                                                   "directives" 
; Delete above title block and comments to developers prior to 
; installing into your application - actual apps header below
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>        <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

;EXT_APP equ 1   ;This definition is required of all apps
cseg            ;This linker directive is required of all apps.
;
;***********************************************************************
;*  This is the "Application Header Definition" area, required for all 
;*  apps. Note than the data bytes with leading nibble  = hex 1 thru F 
;*  have a leading zero to direct certain assemblers this is a data 
;*  field and not a variable name. The size of the Application Header
;*  is 128 bytes with the last 32 or so bytes reserved as a pad for 
;*  modifications within various fields.  This will allow your 
;*  application to be located at a specific position in FLASH - 128 
;*  bytes from the physical start of the page. For instance, in the 
;*  with a single additional application, it is located in page 15h. -  
;*  the header is at address 15 x 4000 or 54000; your program starts at 
;*  an offset of 080h from there.   
;***********************************************************************

                   		;Field code = 080h:  Field codes are delimiters 
                        ; of the various datums in the header file. 
 .db 080h,0Fh    		;Field:Program Length      where "length" = 0Fh 
						; and the "Fh"  tells us four bytes follow 
 .db 00h,00h,00h,00h 	;Length= 4 bytes in size = (0 or N/A for 
                        ; unsigned apps) For signed apps the length is 
                        ; TBD/TBA 
; 
 .db 080h,012h   		;Field:Program type    Prgm.type, in 2nd byte is 12h.
						; The "2" defines the number of bytes in the keyfile 
						; name (which are the next 2 bytes). As illustrated in 
						; the Demo's Keyfile example from the Developer's Guide,
						; the keyfile example was 0104; two bytes in size.
;
 .db  01h,04h                          ; Prgm.Type = Keycode file  (= Shareware for TI-83Plus)
;  
;
;	.db $80,$13			;The "3" tells num of bytes in Key.
;       .db     $01,$9F,$04             ;My Key Number "19F04".
;        .db $9f,$04
;       .db $80,$13                     ;The "3" tells num of bytes in Key.
;	.db $01,$BF,$04		;Andrew's Key Number "1BF04".

;    In the Developers Guide the "type" field is the keyfile code,  
;    ...."0104". For a keyfile of  5 or 6 characters, for example: 
;    "17F04", one must change the "2" to a "3" to inform your calculator
;    that the Keycode file's datafield is 5 or 6 hex characters in size.  
;    NOTE:  Use the above example's "01h 04h"  keycode when working on 
;    the TI-83Plus debugger. The below example is only for when you 
;    sign and install the app on your calculator and you have a 3 byte
;    keycode (5 or 6 digits).
;  
;    ADDITIONAL NOTE(2): WHEN SUBMITTING APPLICATIONS TO TI TO BE SIGNED 
;    PLEASE RESTORE THE PROGRAM TYPE AND KEYCODE TO THE HEADER's 
;    DEFAULTS OF:   db  080h, 012h
;	            db   01h, 04h
;    AND RESTORE THE 00h "PADDING" BYTE AT THE END OF THE HEADER. 
;    ALSO, RECOMPILE THE APPLICATION TO GENERATE A NEW HEX FILE.   
;
; db 080h,013h	    ; <-- Use this if your keyfile is 3 bytes in size
;                   ;     eg, a 5 or 6 hex-digit keyfile.
; db  01h,7Fh,04h   ;  An example 5 character keycode of 17F04  	
;
;*********************************************************************
;*     NOTE: We must modify also the "pad" at the end of this 
;*     Application Header file, to subtract the extra byte used here.
;*     This will maintain the  header file size at 128 bytes. 
;*********************************************************************
;
					;Field: App ID   Field code = 080h and App ID = "21h" 
 .db 080h,021h		; "2" = 2nd data field, 1 for one byte in size.
;
                    ;ID = 1 and it's one byte in size, as denoted in the 
        			; App ID's second nibble("1").This field is typically
                	; incremented on shareware or authenicated apps for 
 .db   01h			; major revisions,i.e:  1.x to 2.0; 2.n to 3.0, etc.
;
; .db "HELfP"					;Field: App Build       App Build = "31h" ; the
					; "3"  tells us it's build data; the "1" tells us the
 .db 080h,031h		;  Build data is "1" byte in size
;
                	;Build = 1 (always = 1 for development); is the 
                	; configuration control for minor revisions, eg: 
 .db   01h			; 1.1 to 1.2, or X.m to X.(m+1)
; .db "ME!!!!"
;
;*
;***********************************************************************
;*    Regarding the below commented-out App Name, if you wanted to  
;*    name your application "InMotion" rather than "Hello", you could.
;*    You would use values as shown in commented out example below. 
;*    NOTE: Regardless of App Header files' "Pad bytes", app name
;*          must equal 8 characters - no  more and no less !  Refer 
;*          to the Developer's Guide examples of User-Named Programs,
;*          (and Used-Named AppVars, and Group). 
;***********************************************************************
;
; db 080h,048h	      ;Field:App Name   where App Name code = "4"h; the 
;                                       "8" tell us 8 bytes for size of
;                                       name
;  
; dB 049h,06Eh,04Dh,06Fh,074h,069h,06Fh,06Eh	; App Name = InMotion   
;
;***********************************************************************
;*    In the Hello example, the App Name is: 
;*   "LcapH, Le,Ll,Ll,Lo,Lspace,Lspace,Lspace" 
;* These are equates (refer to the Ti83plus.inc file) having values of: 
;*    048h, 065h, 06ch, 06ch, 06fh, 020h,020h,020h 
;*    which spells out "Hello^^^"   (showing ^^^ to represent 3 blanks)
;***********************************************************************
;
 .db 080h,048h			;Field:App Name   and App Name ="48h"
;
; .db 048h,065h,06ch,06ch,06fh,020h,020h,020h  ; App Name="Hello   "
;                                            ; and MUST be 8 characters
  .db "PortMon "
;
 .db 080h,081h			;Field: App Pages    App pages code = "8" &
			 			;  the "1" infers one byte following which 
						;  tells us the number of pages of the app.        
 .db 01h				;App Pages = 1   eg, a 1-page application
;
 .db 080h,090h			;No default splash screen
;
 .db 03h,026h,09h,04h,04h,06fh,01bh,80h   ;Field: Date stamp- 5/12/1999
                                         ;(No decode of the Date Stamp)
;
 .db 02h,0dh,040h              ;Dummy-encrypted TI date stamp signature
		                       ;(No decode of encrypted date stamp
                               ; signature) 
;                                             
; ==== Below 64 bytes of data are p/o application data  ===== 
;
 .db 0a1h ,06bh ,099h ,0f6h ,059h ,0bch ,067h 
 .db 0f5h ,085h ,09ch ,09h ,06ch ,0fh ,0b4h ,03h ,09bh ,0c9h 
 .db 03h ,032h ,02ch ,0e0h ,03h ,020h ,0e3h ,02ch ,0f4h ,02dh 
 .db 073h ,0b4h ,027h ,0c4h ,0a0h ,072h ,054h ,0b9h ,0eah ,07ch 
 .db 03bh ,0aah ,016h ,0f6h ,077h ,083h ,07ah ,0eeh ,01ah ,0d4h 
 .db 042h ,04ch ,06bh ,08bh ,013h ,01fh ,0bbh ,093h ,08bh ,0fch 
 .db 019h ,01ch ,03ch ,0ech ,04dh ,0e5h ,075h 
;
 .db 80h,7Fh		;Field:  Program Image length 
 .db   0,0,0,0		;Length=0, N/A
;
 .db   0,0,0,0		;Reserved / pad  
 .db   0,0,0,0		;Reserved / pad
 .db   0,0,0,0		;Reserved / pad
 .db   0,0,0,0		;Reserved / pad (use if you have a 4 hex character
                    ;                 sized keyfile)
; .db   0,0,0
;  
;   If your keyfile was a five or six hex characters; (eg, 17F04) delete 
;   delete a single "0" from the last line of pad; eg: 
;       change from:   db    0,0,0,0 
;                to:   db    0,0,0
; 	
;****************************************************************
;    >>>>>       End of Header Data       <<<<< 
;****************************************************************
;
; <Your program's "Startapp:" label would go here. Note that the 
;   "Startapp:" label must be used; no custom labels, eg, "Main" 
;   or "Start_AppName" is allowed.  >


;Can't believe I'm using this template...


#define bcall(label) RST 28h \ .dw label
#define bjump(label) call 50h \ .dw label



;Mem page format;
;2 - ID
;1 - CurSamps
;1 - MaxSamps
;1 - LowPort
;1 - NumPorts

tempnum = appbackupscreen
curhighport = appbackupscreen+2
curlowport = appbackupscreen+3
curmaxsamps = appbackupscreen+4
cursamp = appbackupscreen+5
curport = appbackupscreen+6
curnumsamps = appbackupscreen+7
curnumports = appbackupscreen+8
curnumchangeports = appbackupscreen+9
curmask = appbackupscreen+10
command = appbackupscreen+11
port = appbackupscreen+13 ;believe it or not, this is used by the string input
textbuf = appbackupscreen+50
code = appbackupscreen+150
	jp	init

Init:
	in	a,(2)
	add	a,a
	jp	nc,NotSE
	ld	a,1
	out	(20h),a
	call	DispMainMenu
	call	igetkey
	cp	k1
	jp	z,HookStart
	cp	k2
	jp	z,IntStart
	cp	k3
	jp	z,HookStop
	cp	k4
	jr	z,Exec
	cp	k5
	jp	z,HookDisp
	cp	k6
	jp	z,HookChanges
	cp	kclear
	jr	z,donedonedone
	cp	k7
	jp	nz,Init
donedonedone:
	bjump(_JForceCmdNoChar)


execmenutxt:
	.db	"Exec            "
	.db	"1) Program      "
	.db	"2) BCALL        "
	.db	"3) Code         "
	.db	"4) Back",0

Exec:
	bcall(_clrlcdfull)
	bcall(_homeup)
	ld	hl,ExecMenuTXT
	call	puts
	bcall(_newline)
execloop:
	call	igetkey
	cp	k1
	jp	z,execprog
	cp	k2
	jp	z,execbcall
	cp	k4
	jp	z,Init
	cp	k3
	jr	nz,execloop

execcode:
	set	appautoscroll,(iy+appflags)
	ld	hl,codesizetxt
	call	puts
	call	gethexa
	push	af
	bcall(_newline)
	pop	af
	ld	b,a
	ld	hl,code
execcodeloop:
	push	hl
	push	bc
	call	gethexa
	pop	bc
	pop	hl
	ld	(hl),a
	inc	hl
	djnz	execcodeloop
	ld	(hl),0C9h
	call	code
	jp	Init
execbcall:
	ld	hl,bcalltxt
	call	puts
	call	gethexa
	push	af
	call	gethexa
	pop	hl
	ld	l,a
	ld	(code+1),hl
	ld	a,0EFh
	ld	(code),a
	ld	a,0C9h
	ld	(code+3),a
	call	code
	jp	Init
execprog:
	ld	hl,NameTXT
	call	puts
	ld	b,8
	call	gettext
	ld	hl,textbuf-1
	ld	(hl),6
	rst	20h
	bcall(_chkfindsym)
	jp	c,Init
	inc	b
	dec	b
	jp	nz,Init
	ex	de,hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	inc	hl
	ld	a,84h
	cp	(hl)
	jp	nz,Init
	inc	hl
	ld	a,'U'
	cp	(hl)
	jp	nz,Init
	inc	hl
	ld	de,code
	ldir	
	call	code
	jp	init
NameTXT:
	.db	"Name:",0
bcalltxt:
	.db	"BCALL:",0
codesizetxt:
	.db	"Size:",0

StartStuffInt:
	ld	hl,MaskTXT
	call	Puts
	call	GetHeXA
	ld	(curmask),a
	bcall(_newline)
StartStuff:

	ld	hl,LowPortTXT
	call	PutS
	call	GetHexA
	ld	(curLowPort),a
	bcall(_newline)
	ld	hl,HighPortTXT
	call	PutS
	call	GetHexA
	ld	(curHighPort),a
	bcall(_NewLine)
	ld	hl,MaxSampsTXT
	call	PutS
	call	GetHexA
	ld	(curMaxSamps),a
	bcall(_NewLine)
	di
	ld	a,7h
	out	(5),a
	ld	hl,0C000h
	ld	(hl),84h
	inc	hl
	ld	(hl),'U'
	inc	hl
	ld	(hl),0
	inc	hl
	ld	a,(CurMaxSamps)
	ld	(hl),a
	inc	hl
	ld	a,(curLowPort)
	ld	(hl),a
	inc	hl
	ld	b,a
	ld	a,(CurHighPort)
	sub	b
	inc	a
	ld	(hl),a
	inc	hl
	ld	a,(curmask)
	ld	(0FFFFh),a
	xor	a
	out	(5),a
	ret
	
IntStart:
	im	1
	bcall(_ClrLCDFull)
	bcall(_HomeUp)
	call	StartStuffInt
	bcall(528Dh)
	ld	de,8A8Ah
	ld	hl,IntCode
	ld	bc,IntCodeSize
	ldir
	ld	a,d
	ld	hl,8B00h
	ld	(hl),a
	ld	de,8B01h
	ld	bc,256
	ldir
	inc	a
	ld	i,a
	im	2
	jr	StartCont

	

	
HookStart:
	im	1
	bcall(_ClrLCDFull)
	bcall(_HomeUp)
	call	StartStuff

	ld	hl,USBHook
	in	a,(6)
	bcall(528Ah)
StartCont:
	ld	hl,USBStartTXT
	call	PutS
	call	igetkey
	jp	Init




IntCode:
	exx
	ex	af,af'
	ld	a,7h
	out	(5),a
	ld	a,(0FFFFh)
	ld	b,a
	in	a,(4)
	and	b
	jr	z,retnzi
	ld	hl,0C002h
	ld	a,(hl)
	or	a
	jr	nz,notfirst
	inc	(hl)
notfirst:
	inc	hl
	ld	b,(hl)
	cp	b
	jr	z,retnzi
	dec	hl
	push	hl
	inc	hl
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	inc	hl
	ld	e,a
	push	de
	push	hl
	push	bc
	ld	h,b
	ld	l,a
	bcall(_htimesl)
	pop	bc
	pop	de
	add	hl,de	
	pop	de		;e = current sample 
	ld	d,b		;d = number of ports
intLoop:
	ld	a,84h
	cp	c
	jr	z,intloopskip
	ld	a,82h
	cp	c
	jr	z,intloopskip
	ld	a,0A0h
	cp	c
	jr	z,intLoopskip
	ld	a,8Ch
	cp	c
	jr	z,intloopskip
	inc	a
	cp	c
	jr	z,intloopskip
	in	a,(c)
intLoopSkip:
	ld	(hl),a
	inc	e
	dec	e
	jr	z,intloopskip2
	push	de
	ld	e,d
	ld	d,0
	or	a
	sbc	hl,de
	cp	(hl)
	push	af
	add	hl,de
	pop	af
	pop	de
	jr	z,intloopskip2
	ex	de,hl
	pop	hl
	inc	(hl)
	push	hl
	ex	de,hl
	ld	e,0
intloopskip2:

	inc	hl
	inc	c
	djnz	intloop
	pop	hl
retnzi:
	xor	a
	out	(5),a
	ld	b,1
	exx
	ex	af,af'
	jp	38h
IntCodeSize = $-IntCode

USBHook:
	add	a,e
	ld	a,87h
	out	(7),a
	ld	hl,8002h
	ld	a,(hl)
	inc	hl
	ld	b,(hl)
	cp	b
	jp	z,retnz
	dec	hl
	inc	(hl)
	inc	hl
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	inc	hl
	push	hl
	push	bc
	ld	l,a
	ld	h,b
	bcall(_htimesl)
	pop	bc
	pop	de
	add	hl,de
SampLoop:
	ld	a,84h
	cp	c
	jr	z,samploopskip
	ld	a,82h
	cp	c
	jr	z,samploopskip
	ld	a,0A0h
	cp	c
	jr	z,SampLoopskip
	in	a,(c)
sampLoopSkip:
	ld	(hl),a
	inc	hl
	inc	c
	djnz	SampLoop
retnz:
	ld	a,81h
	out	(7),a
	ld	b,1
	ret
retnzforreal:
	xor	a
	inc	a
	ret

HookStop:
	im	1
	bcall(528Dh)
	bcall(_clrlcdfull)
	bcall(_homeup)
	ld	hl,usbstopTXT
	call	puts
	call	igetkey
	jp	Init


HookValid:
	ld	hl,8000h
	call	GetVal
	cp	84h
	ret	nz
	inc	hl
	call	GetVal
	cp	'U'
	ret	nz
	inc	hl
	call	GetVal
	or	a
	jp	z,retnzforreal
	ld	(curnumsamps),a
	inc	hl
	inc	hl
	call	GetVal
	ld	(curlowport),a
	ld	(curport),a
	inc	hl
	call	GetVal
	ld	(curnumports),a
	xor	a
	ld	(cursamp),a
	ret

GetNumChanges:
	ld	a,(curNumPorts)
	ld	b,a
	ld	a,(curlowport)
	ld	c,0
GetNumChangesLoop:
	call	IsChangePort
	jr	z,GetNumChangesNo
	inc	c
GetNumChangesNo:
	inc	a
	djnz	GetNumChangesLoop
	ld	a,c
	ld	(curNumChangePorts),a
	or	a
	ret

IsChangePort:
	push	bc
	push	af
	push	af
	ld	a,(curport)
	ld	h,a
	ld	a,(cursamp)
	ld	l,a
	pop	af
	push	hl
	ld	(curport),a
	ld	a,(curnumsamps)
	ld	b,a
	dec	a
	jr	z,IsChangePortDone
	ld	(cursamp),a
	ld	a,(curport)
	call	GetPortVal
	ld	c,a
IsChangePortLoop:
	djnz	IsChangePortDo
IsChangePortDone:
	pop	hl
	ld	a,l
	ld	(cursamp),a
	ld	a,h
	ld	(curport),a
	pop	bc
	ld	a,b
	pop	bc
	ret
IsChangePortDo:
	ld	a,b
	dec	a
	ld	(cursamp),a
	ld	a,(curport)
	call	GetPortval
	cp	c
	jr	nz,IsChangePortDone
	jr	IsChangePortLoop
	
DispChangeScreen:
	res	appautoscroll,(iy+appflags)
	ld	a,(curport)
	push	af
	bcall(_clrlcdfull)
	bcall(_homeup)
	ld	a,' '
	call	putc
	call	putc
	call	putc	
	ld	a,(curnumsamps)
	ld	c,a
	ld	a,(cursamp)
	ld	b,4
DispChangesSampLoop:
	call	DispHexA
	inc	a
	push	af
	ld	a,' '
	call	putc
	pop	af
	cp	c
	jr	z,DispChangesSampDone
	djnz	DispChangesSampLoop
DispChangesSampDone:


	ld	a,(curport)
	ld	c,a
	ld	a,(curlowport)
	dec	a
	inc	c
DCPL2:
	inc	a
	call	IsChangePort
	jr	z,DCPL2
	dec	c
	jr	nz,DCPL2
;	dec	a		;a = first realport to display
DCPL2Done:
	ld	b,7
DispChangesPortLoop:
	inc	b
	call	IsChangePort
	ld	(curport),a

	jr	z,DispChangesPortNo
	dec	b
	push	af
	push	bc
	bcall(_newline)
	pop	bc
	pop	af
	call	DispHexA

	ld	a,':'
	call	putc
	ld	a,(cursamp)
	push	af
	push	bc
	ld	b,4
DispChangesPortLoop3:
	ld	a,(curport)
	call	getportval
	call	disphexa
	ld	a,' '
	call	putc
	ld	hl,cursamp
	inc	(hl)
	ld	a,(curnumsamps)
	cp	(hl)
	jr	z,DispChangesPortLoop3Done
	djnz	DispChangesPortLoop3
DispChangesPortLoop3Done:
	pop	bc
	pop	af
	ld	(cursamp),a
DispChangesPortNo:
	ld	a,(curnumports)
	ld	c,a
	ld	a,(curlowport)
	add	a,c
	ld	c,a
	ld	a,(curport)
	inc	a
	cp	c
	jr	z,DispChangesPortDone
	djnz	DispChangesPortLoop
DispChangesPortDone:
	pop	af
	ld	(curport),a
	ret

HookChanges:
	call	HookValid
	jp	nz,Init	
	ld	a,(curnumsamps)
	sub	2
	jp	c,Init
	call	GetNumChanges
	jp	z,Init
	xor	a
	ld	(curport),a
	ld	(cursamp),a
HookChangeLoop:
	call	DispChangeScreen
	call	igetkey
	cp	kup
	jr	z,HookChangeUp
	cp	kdown
	jr	z,HookChangeDown
	cp	kleft
	jr	z,HookChangeLeft
	cp	kright
	jr	z,HookChangeRight
	cp	kclear
	jp	z,Init
	cp	kquit
	jr	nz,HookChangeLoop
	jp	Init
HookChangeUp:
	ld	a,(curport)
	add	a,0FFh
	ccf
	adc	a,0
	ld	(curport),a
	jr	HookChangeLoop
HookChangeDown:
	ld	a,(curport)
	add	a,7+1
	ld	b,a
	ld	a,(curnumchangeports)
	cp	b
	jr	c,HookChangeLoop
	ld	a,b
	sub	7
	ld	(curport),a
	jr	HookChangeLoop
HookChangeLeft:
	ld	a,(cursamp)
	or	a
	jr	z,HookChangeLoop
	dec	a
	ld	(cursamp),a
	jr	HookChangeLoop
HookChangeRight:
	ld	a,(curnumsamps)
	sub	4
	jr	c,HookChangeLoop
	ld	b,a
	ld	a,(cursamp)
	cp	b
	jr	z,HookChangeLoop
	inc	a
	ld	(cursamp),a
	jr	HookChangeLoop
	
HookDisp:
	call	HookValid
	jp	nz,Init
HookDispLoop:
	call	DispScreen
	call	igetkey
	cp	kup
	jp	z,HookDispUp
	cp	kdown
	jp	z,HookDispDown
	cp	kleft
	jp	z,HookDispLeft
	cp	kright
	jp	z,HookDispRight
	cp	kClear
	jp	z,Init
	cp	kQuit
	jp	z,Init
	jp	HookDispLoop
HookDispUp:
	ld	a,(curlowPort)
	ld	b,a
	ld	a,(curport)
	cp	b
	jp	z,HookDispLoop
	sub	4
	ld	(curport),a
	jp	HookDispLoop
HookDispDown:
	ld	a,(curlowport)
	ld	b,a
	ld	a,(curnumports)
	add	a,b
	ld	c,a
	ld	a,(curport)
	add	a,7*4
	cp	c
	jp	nc,HookDispLoop
	sub	7*4-4
	ld	(curport),a
	jp	HookDispLoop
HookDispRight:
	ld	a,(curnumsamps)
	ld	b,a
	ld	a,(cursamp)
	inc	a
	cp	b
	jp	z,HookDispLoop
	ld	(cursamp),a
	jp	HookDispLoop
HookDispLeft:
	ld	a,(cursamp)
	or	a
	jp	z,HookDispLoop
	dec	a
	ld	(cursamp),a
	jp	HookDispLoop
DispScreen:
	bcall(_clrlcdfull)
	bcall(_homeup)
	ld	hl,cursamptxt
	call	puts
	ld	a,(cursamp)
	call	disphexa
	bcall(_newline)
	ld	a,(curlowport)
	ld	b,a
	ld	a,(curnumports)
	add	a,b
	ld	c,a		;first off limit port
	ld	d,7		; loops of port lines
	ld	a,(curport)
	ld	e,a		;cur port
DispScreenLoop:
	ld	a,e
	call	disphexa
	ld	a,':'
	call	putc
	ld	b,4
dispscreenLoop2:
	ld	a,e
	cp	c
	ret	z
	call	GetPortval
	call	DispHexA
	ld	a,' '
	call	putc
	inc	e
	djnz	dispScreenLoop2
	push	bc
	push	de
	bcall(_newline)
	pop	de
	pop	bc
	dec	d
	jp	nz,DispScreenLoop
	ret
GetPortval:
	push	bc
	push	de
	ld	b,a
	ld	a,(curlowport)
	ld	c,a
	ld	a,b
	sub	c
	push	af
	ld	a,(curnumports)
	ld	h,a
	ld	a,(cursamp)
	ld	l,a
	bcall(_htimesl)
	pop	af
	ld	e,a
	ld	d,0
	add	hl,de
	ld	de,8006h
	add	hl,de
	di	
	ld	a,87h
	out	(7),a
	ld	b,(hl)
	ld	a,81h
	out	(7),a
	ld	a,b
	pop	de
	pop	bc
	ei
	ret
		
			


GetVal:
	di	
	ld	a,87h
	out	(7),a
	ld	b,(hl)
	ld	a,81h
	out	(7),a
	ld	a,b
	ei
	ret
	
avname:
	.db	AppVarObj,"USBTEST"
	

	.db	200
	bcall(_clrlcdfull)
	bcall(_homeup)
	.ds	200
	.db	3	
DispMainMenu:
	bcall(_ClrLCDFull)
	bcall(_HomeUP)
	ld	hl,MainMenuTXT
	jp	Puts

MainMenuTXT:
	.db	"Port Monitor    "
	.db	"1) Setup Hook   "
	.db	"2) Setup Int    "
	.db	"3) Disable Mon  "
	.db	"4) Exec Code    "
	.db	"5) View Data    "
	.db	"6) View Changes "
	.db	"7) Quit",0

LowPortTXT:
	.db	"Low Port:",0
HighPortTXT:
	.db	"High Port:",0
MaxSampsTXT:
	.db	"Max Samps:",0
MaskTXT:
	.db	"Port 4 Mask:",0
USBStartTXT:
	.db	"Monitor On",0
USBStopTXT:
	.db	"Monitor Off",0
CurSampTXT:
	.db	"Cur Samp:",0

GetHexA:
		; lets user input an 8 bit number in hexadecimal
		; prompt is at currow,curcol
		; number is returned in a
	set	curable,(iy+curflags)
	ld	b,2
	ld	hl,tempnum
getnumhloop:
	push	bc
	push	hl
	call	igetkey
	pop	hl
	pop	bc
	cp	2
	jp	nz,gnhnotback
	ld	a,b
	cp	2
	jp	z,gnhnotback
	ld	a,' '
	bcall(_putmap)
	ld	hl,curcol
	dec	(hl)
	jp	gethexa

gnhnotback:
	sub	142
	cp	10
	jp	c,gnhnumpressed
	sub	12
	cp	6
	jp	c,gnhletpressed
	jp	getnumhloop
gnhnumpressed:
	ld	(hl),a
	inc	hl
	add	a,48
	call	putc
	djnz	getnumhloop
	jp	gnhdone
gnhletpressed:
	add	a,10
	ld	(hl),a
	inc	hl
	add	a,55
	call	putc
	djnz	getnumhloop
gnhdone:
	dec	hl
	ld	b,(hl)
	dec	hl
	ld	a,(hl)
	rlca
	rlca
	rlca
	rlca
	or	b
	res	curable,(iy+curflags)
	ret



putc:
        push    hl
        push    bc
        bcall(_putc)
        pop     bc
        pop     hl
        ret


PutS:
	ld	a,(hl)
	inc	hl
	or	a
	ret	z
	call	putc
	jp	Puts


disphexa:
	push	af
        push    hl
        push    bc
        push    af
        rrca
        rrca
        rrca
        rrca
        call    dispha
        pop     af
        call      dispha
        pop     bc
        pop     hl
	pop	af
        ret
dispha:
        and     15
        cp      10
        jp      nc,dhlet
        add     a,48
        jp      dispdh
dhlet:
        add     a,55
dispdh:
	bcall(_putc)
        ret


NotSE:
	bcall(_clrlcdfull)
	bcall(_homeup)
	ld	hl,SETXT
	call	puts
	call	igetkey
	bjump(_jforcecmdnochar)
SETXT:
	.db	"This program    "
	.db	"requires an SE  "
	.db	" or 84 calc",0


MultHE:
	ld	d,0
	ld	l,0
	ld	b,8
MultHELoop:
	add	hl,hl
	jr	nc,NoAdd
	add	hl,de
NoAdd:
	djnz	MultHELoop
	ret	

igetkey:
	res	oninterrupt,(iy+onflags)
	bcall(_getkey)
	ret








gettext:
		; gets input from user as text
		; input is at cursor
		; b = max length of string (must be <= 255)
	
	set	appautoscroll,(iy+appflags)
	set	curable,(iy+curflags)
	ld	hl,(currow)
	ld	(command),hl
	ld	hl,textbuf
	ld	a,b
	ld	(port),a
gettextloop:
	push	hl
	push	bc
	call	igetkey
	pop	bc
	pop	hl
	cp	$ca
	jr	z,gtques
	cp	$8b
	jr	z,gtcomma
	cp	02
	jr	z,gtbackspace
	cp	05
	jp	z,gtfinished
	sub	$8e
	jr	c,gettextloop
	cp	10
	jr	c,gtnumber
	sub	$99-$8e
;	jr	c,gettext
	jr	c,gettextloop
	cp	27
	jr	c,gtletter
	cp	27
	jr	z,gtx
	jr	gettextloop
gtbackspace:
	ld	a,(port)
	cp	b
	jr	z,gettextloop
	inc	b
	dec	hl
	ld	a,' '
	bcall(_putmap)
	ld	a,(curcol)
	or	a
	jr	z,gtbsneedline
	dec	a
	ld	(curcol),a
	ld	a,' '
	bcall(_putmap)
	jp	gettextloop
gtbsneedline:
	ld	a,15
	ld	(curcol),a
	ld	a,(currow)
	dec	a
	ld	(currow),a
	ld	a,' '
	bcall(_putmap)
	jp	gettextloop
gtnumber:
	add	a,48
	ld	(hl),a
	inc	hl
	bcall(_putc)
gtndjnz:
	djnz	gettextloop
	jr	gtfinished
gtcomma:
	ld	a,','
	jr	gtlloadval
gtletter:
	or	a
	jr	z,gtspace
	add	a,64
gtlloadval
	ld	(hl),a
	inc	hl
	bcall(_putc)
	jr	gtndjnz
gtques:
	ld	a,'?'
	jr	gtlloadval
gtx:
	ld	a,'X'
	jr	gtlloadval
gtspace:
	ld	a,32
	jr	gtlloadval

gtfinished:
	ld	(hl),0
	ld	a,' '
	bcall(_putmap)
	res	curable,(iy+curflags)
	res	appautoscroll,(iy+appflags)
	ret









.end
